@@ -327,6 +327,10 @@ Package *Backend::packageForFile(const QString &file) const
 {
     Q_D(const Backend);
 
+    if (file.isEmpty()) {
+        return 0;
+    }
+
     Q_FOREACH (Package *package, d->packages) {
         if (package->installedFilesList().contains(file)) {
             return package;
@@ -410,6 +414,12 @@ qint64 Backend::downloadSize() const
     // Raw size, ignoring already-downloaded or partially downloaded archives
     qint64 downloadSize = d->cache->depCache()->DebSize();
 
+    // If downloadSize() is called during a cache refresh, there is a chance it
+    // will do so at a bad time and produce an error. Discard any errors that
+    // happen during this function, since they will always be innocuous and at
+    // worst will result in the less accurate DebSize() number being returned
+    _error->PushToStack();
+
     // If possible, get what really needs to be downloaded
     pkgAcquire fetcher;
     pkgPackageManager *PM = _system->CreatePM(d->cache->depCache());
@@ -418,6 +428,9 @@ qint64 Backend::downloadSize() const
     }
     delete PM;
 
+    _error->Discard();
+    _error->RevertToStack();
+
     return downloadSize;
 }
 
