@@ -45,12 +45,12 @@
 
 // Qt includes
 #include <QtCore/QFile>
-#include <QtCore/QProcess>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusMessage>
 
 #define RAMFS_MAGIC     0x858458f6
 
+#include "../debfile.h"
 #include "debug.h"
 #include "qaptauthorization.h"
 #include "workeracquire.h"
@@ -452,6 +452,141 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
     installProgress = 0;
 }
 
+void QAptWorker::downloadArchives(const QStringList &packageStrings, const QString &dest)
+{
+    if (!initializeApt()) {
+        emit workerFinished(false);
+        return;
+    }
+
+    emit workerStarted();
+
+    initializeStatusWatcher();
+    pkgAcquire fetcher;
+    fetcher.Setup(m_acquireStatus);
+
+    pkgIndexFile *index;
+
+    foreach (const QString &packageString, packageStrings) {
+        pkgCache::PkgIterator iter = m_cache->depCache()->FindPkg(packageString.toStdString());
+
+        if (!iter) {
+            // Package not found
+            continue;
+        }
+
+        pkgCache::VerIterator ver = (*m_cache->depCache()).GetCandidateVer(iter);
+
+        if (!ver || !ver.Downloadable() || !ver.Arch()) {
+            // Virtual package or not downloadable or broken
+            continue;
+        }
+
+        // Obtain package info
+        pkgCache::VerFileIterator vf = ver.FileList();
+        pkgRecords::Parser &rec = m_records->Lookup(ver.FileList());
+
+        // Try to cross match against the source list
+        if (!m_cache->list()->FindIndex(vf.File(), index)) {
+            continue;
+        }
+
+        string fileName = rec.FileName();
+        string MD5sum = rec.MD5Hash();
+
+        if (fileName.empty()) {
+            //TODO: Error
+            return;
+        }
+
+        new pkgAcqFile(&fetcher,
+                       index->ArchiveURI(fileName),
+                       MD5sum,
+                       ver->Size,
+                       index->ArchiveInfo(ver),
+                       ver.ParentPkg().Name(),
+                       dest.toStdString(), "");
+    }
+
+    emit workerEvent(QApt::PackageDownloadStarted);
+
+    if (fetcher.Run() != pkgAcquire::Continue) {
+        // Our fetcher will report errors for itself, but we have to send the
+        // finished signal
+        emit workerFinished(false);
+        return;
+    }
+
+    emit workerEvent(QApt::PackageDownloadFinished);
+    emit workerFinished(true);
+}
+
+void QAptWorker::installDebFile(const QString &fileName)
+{
+    if (!initializeApt()) {
+        emit workerFinished(false);
+        return;
+    }
+
+    emit workerStarted();
+
+    QApt::DebFile deb(fileName);
+
+    QString arch = deb.architecture();
+
+    if (arch != QLatin1String("all") &&
+        arch != QString::fromStdString(_config->Find("APT::Architecture", ""))) {
+        // TODO report what arch was provided vs needed
+        emit errorOccurred(QApt::WrongArchError, QVariantMap());
+        emit workerFinished(false);
+        return;
+    }
+
+    if (!QApt::Auth::authorize(QLatin1String("org.kubuntu.qaptworker.commitChanges"), message().service())) {
+        emit errorOccurred(QApt::AuthError, QVariantMap());
+        emit workerFinished(false);
+        return;
+    }
+
+    m_dpkgProcess = new QProcess(this);
+    QString program = QLatin1Literal("dpkg") %
+                      QLatin1Literal(" -i ") % fileName;
+    setenv("PATH", "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin", 1);
+    setenv("DEBIAN_FRONTEND", "passthrough", 1);
+    setenv("DEBCONF_PIPE", "/tmp/qapt-sock", 1);
+    m_dpkgProcess->start(program);
+    connect(m_dpkgProcess, SIGNAL(started()), this, SLOT(dpkgStarted()));
+    connect(m_dpkgProcess, SIGNAL(readyRead()), this, SLOT(updateDpkgProgress()));
+    connect(m_dpkgProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
+            this, SLOT(dpkgFinished(int, QProcess::ExitStatus)));
+
+}
+
+void QAptWorker::dpkgStarted()
+{
+    emit workerEvent(QApt::DebInstallStarted);
+}
+
+void QAptWorker::updateDpkgProgress()
+{
+    QString str = m_dpkgProcess->readLine();
+
+    emit debInstallMessage(str);
+}
+
+void QAptWorker::dpkgFinished(int exitCode, QProcess::ExitStatus exitStatus)
+{
+    Q_UNUSED(exitCode);
+
+    if (!exitStatus) {
+        emit workerEvent(QApt::DebInstallFinished);
+    }
+
+    emit workerFinished(!exitStatus);
+    delete m_dpkgProcess;
+    m_dpkgProcess = 0;
+}
+
 void QAptWorker::answerWorkerQuestion(const QVariantMap &response)
 {
     emit answerReady(response);
@@ -514,3 +649,23 @@ bool QAptWorker::writeFileToDisk(const QString &contents, const QString &path)
 
     return false;
 }
+
+bool QAptWorker::copyArchiveToCache(const QString &archivePath)
+{
+    if (!QApt::Auth::authorize(QLatin1String("org.kubuntu.qaptworker.writeFileToDisk"), message().service())) {
+        emit errorOccurred(QApt::AuthError, QVariantMap());
+        return false;
+    }
+
+    initializeApt();
+    QString cachePath = QString::fromStdString(_config->FindDir("Dir::Cache::Archives"));
+    // Filename
+    cachePath += archivePath.right(archivePath.size() - archivePath.lastIndexOf('/'));
+
+    if (QFile::exists(cachePath)) {
+        // Already copied
+        return true;
+    }
+
+    return QFile::copy(archivePath, cachePath);
+}
